/*
 * An interactive program to manage a (local or remote) object space.
 * You can use in, out, read (and print) from your console.
 * Type end when you're tired enough :)
 *
 * Usage: java InteractiveClient [-r [hostname [port]]]
 * 	-r: use a remote object server
 *  hostname: the host which runs the remote object server 
 *		(defaults to localhost)
 *	port: the port for the remote object server 
 *		(defaults to ObjectNetConst.DEFAULT_PORT)
 *
 * (C) 1996 Davide Rossi
 */

import java.io.*;
import jada.*;
import jada.net.*;

public class InteractiveClient implements JadaNetConst {

	//program's entry point
	public static void main(String args[]) {

		//the object space
		ObjectSpace object_space=null;
		//weather it is local or remote
		boolean remote=false;
		//where the remote object server is
		String objecthost="localhost";
		//the remote port
		int port=DEFAULT_PORT;

		//input and ouput streams
		PrintStream cout=new PrintStream(System.out);
		DataInputStream cin=new DataInputStream(System.in);

		//check for command line switches
		for(int i=0; i<args.length; i++) {
			if(args[i].equals("-r")) {
				remote=true;
				if(i+1<args.length) {
					i++;
					objecthost=args[i];
				}
				if(i+1<args.length) {
					i++;
					try {
						port=Integer.parseInt(args[i]);
					} catch(NumberFormatException e) {
						cout.println("wrong port format, default used");
					}
				}
			} else if(args[i].equals("-h") || args[i].equals("-help")) {
				cout.println("Usage: java InteractiveClient [-r [hostname [port]]]");
				System.exit(0);
			}
		}

		if(remote) {
			cout.println("Opening a remote object space to "+objecthost);
			object_space=new ObjectClient(objecthost, port);
			JadaSystem.setNetRetryAttempts(0);
			JadaSystem.reportErrors(false);
		} else {
			cout.println("Opening a local object space");
			object_space=new ObjectSpace();
		}

		//the main loop
		boolean cont=true;
		while(cont) {
			cout.print("Tell me, master! "); cout.flush();
			try {
				String input=cin.readLine();
				if(input.startsWith("in_nb")) {
					Object match=new TupleString(input.substring(5));
					Object object=null;
					try {
						object=object_space.in_nb(match);
					} catch (JadaException e) {
						cout.println(e);
					}
					if(object==null) {
						cout.println("failed");
					} else {
						cout.println("in:"+object);
					}
				} else if(input.startsWith("in")) {
					Object match=new TupleString(input.substring(2));
					Object object=null;
					try {
						object=object_space.in(match, 35000);
					} catch (JadaException e) {
						cout.println(e);
					}
					cout.println("in:"+object);
				} else if(input.startsWith("out")) {
					Object  object=new TupleString(input.substring(3));
					try {
						object_space.out(object);
					} catch (JadaException e) {
						cout.println(e);
					}
				} else if(input.startsWith("read_nb")) {
					Object match=new TupleString(input.substring(7));
					Object object=null;
					try {
						object=object_space.read_nb(match);
					} catch (JadaException e) {
						cout.println(e);
					}
					if(object==null) {
						cout.println("failed");
					} else {
						cout.println("read:"+object);
					}
				} else if(input.startsWith("read")) {
					Object match=new TupleString(input.substring(4));
					Object object=null;
					try {
						object=object_space.read(match);
					} catch (JadaException e) {
						cout.println(e);
					}
					cout.println("read:"+object);
				} else if(input.equals("print")) {
					object_space.print();
				} else if(input.equals("reset")) {
					try {
						object_space.reset();
					} catch (JadaException e) {
						cout.println(e);
					}
				} else if(input.equals("close")) {
					try {
						ObjectClient object_client=(ObjectClient)object_space;
						object_client.close();
					} catch(ClassCastException e) {}
				} else if(input.equals("ping")) {
					try {
						ObjectClient object_client=(ObjectClient)object_space;
						if(object_client.isServerAlive()) {
							cout.println("ok");
						} else {
							cout.println("failed");
						}
					} catch(ClassCastException e) {}
				}else if(input.equals("end") || input.equals("exit") ||
				          input.equals("quit") || input.equals("bye")) {
					cont=false;
				}
			} catch(java.io.IOException e) {
				cont=false;
			} catch(Throwable t) {}
		}
	}
}
